<?php

namespace Drupal\activitypub\Controller;

use Drupal\activitypub\Entity\ActivityPubActivityInterface;
use Drupal\activitypub\Entity\ActivitypubActorInterface;
use Drupal\user\UserInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

class InboxController extends BaseController {

  /**
   * Inbox routing callback.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   * @param \Drupal\user\UserInterface $user
   * @param \Drupal\activitypub\Entity\ActivitypubActorInterface $activitypub_actor
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   */
  public function inbox(Request $request, UserInterface $user, ActivitypubActorInterface $activitypub_actor) {
    $status = 400;
    $payload = @json_decode((string)$request->getContent(), TRUE);

    if ($request->getMethod() == 'POST' && !empty($payload)) {
      try {

        // We currently don't verify the signature if it's available since we
        // only accept types which are useful for the user at the moment. The
        // plugins handle them when necessary.

        // Save when the actor is not blocked.
        if (!$this->domainIsBlocked($activitypub_actor->getBlockedDomains(), $payload['actor'])) {

          $object = '';
          if (isset($payload['object'])) {
            if (is_array($payload['object']) && isset($payload['object']['object'])) {
              if ($payload['type'] == 'Accept' && isset($payload['object']['actor'])) {
                $object = $payload['object']['actor'];
              }
              else {
                $object = $payload['object']['object'];
              }
            }
            elseif (is_array($payload['object']) && isset($payload['object']['inReplyTo'])) {
              $object = $payload['object']['inReplyTo'];
            }
            elseif (is_array($payload['object']) && isset($payload['object']['id'])) {
              $object = $payload['object']['id'];
            }
            elseif (is_string($payload['object'])) {
              $object = $payload['object'];
            }
          }

          /** @var \Drupal\activitypub\Entity\Storage\ActivityPubActivityStorage $storage */
          $storage = $this->entityTypeManager()->getStorage('activitypub_activity');
          $values = [
            'uid' => $user->id(),
            'collection' => ActivityPubActivityInterface::INBOX,
            'external_id' => $payload['id'],
            'type' => $payload['type'],
            'actor' => $payload['actor'],
            'object' => $object,
            'payload' => (string) $request->getContent(),
          ];

          /** @var \Drupal\activitypub\Entity\ActivityPubActivityInterface $activity */
          $activity = $storage->create($values);
          $doSave = TRUE;
          $activity->preInboxSave($doSave);
          if ($doSave) {
            $activity->save();
          }

          $status = 202;
        }
        else {
          $status = 403;
        }

      }
      catch (\Exception $e) {
        $this->getLogger('activitypub')->notice('Inbox error: @message - @content', ['@message' => $e->getMessage(), '@content' => (string) $request->getContent()]);
      }
    }

    return new JsonResponse(NULL, $status);
  }

  /**
   * Returns whether the actor is blocked or not.
   *
   * @param $domains
   * @param $actor
   *
   * @return false
   */
  protected function domainIsBlocked($domains, $actor) {
    $blocked = FALSE;

    if (!empty($domains)) {
      $blocked = $this->pathMatcher->matchPath($actor, $domains);
    }

    return $blocked;
  }

}
